<?php

namespace KnitPay\Gateways\Razorpay;

use Pronamic\WordPress\Pay\AbstractGatewayIntegration;

/**
 * Title: Razorpay Integration
 * Copyright: 2020-2021 Knit Pay
 *
 * @author  Knit Pay
 * @version 1.0.0
 * @since   1.7.0
 */
class Integration extends AbstractGatewayIntegration {

	const KNIT_PAY_RAZORPAY_PLATFORM_CONNECT_URL = 'https://razorpay-connect.knitpay.org/';
	const RENEWAL_TIME_BEFORE_TOKEN_EXPIRE       = 1296000; // 15 Days.

	/**
	 * Construct Razorpay integration.
	 *
	 * @param array $args Arguments.
	 */
	public function __construct( $args = array() ) {
		$args = wp_parse_args(
			$args,
			array(
				'id'            => 'razorpay',
				'name'          => 'Razorpay',
				'url'           => 'http://go.thearrangers.xyz/razorpay?utm_source=knit-pay&utm_medium=ecommerce-module&utm_campaign=module-admin&utm_content=',
				'product_url'   => 'http://go.thearrangers.xyz/razorpay?utm_source=knit-pay&utm_medium=ecommerce-module&utm_campaign=module-admin&utm_content=product-url',
				'dashboard_url' => 'http://go.thearrangers.xyz/razorpay?utm_source=knit-pay&utm_medium=ecommerce-module&utm_campaign=module-admin&utm_content=dashboard-url',
				'provider'      => 'razorpay',
				'supports'      => array(
					'webhook',
					'webhook_log',
				),
				// 'manual_url'    => \__( 'http://go.thearrangers.xyz/razorpay', 'pronamic_ideal' ),
			)
		);

		parent::__construct( $args );

		// Actions
		$function = array( __NAMESPACE__ . '\Listener', 'listen' );

		if ( ! has_action( 'wp_loaded', $function ) ) {
			add_action( 'wp_loaded', $function );
		}
	}

	/**
	 * Setup.
	 */
	public function setup() {
		// Display ID on Configurations page.
		\add_filter(
			'pronamic_gateway_configuration_display_value_' . $this->get_id(),
			array( $this, 'gateway_configuration_display_value' ),
			10,
			2
		);

		// Connect/Disconnect Listener.
		$function = array( __NAMESPACE__ . '\Integration', 'update_connection_status' );
		if ( ! has_action( 'wp_loaded', $function ) ) {
			add_action( 'wp_loaded', $function );
		}

		// Get new access token if it's about to get expired.
		add_action( 'knit_pay_razorpay_refresh_access_token', array( $this, 'refresh_access_token' ), 10, 1 );
	}

	/**
	 * Gateway configuration display value.
	 *
	 * @param string $display_value Display value.
	 * @param int    $post_id       Gateway configuration post ID.
	 * @return string
	 */
	public function gateway_configuration_display_value( $display_value, $post_id ) {
		$config = $this->get_config( $post_id );

		return empty( $config->merchant_id ) ? $config->key_id : $config->merchant_id;
	}

	/**
	 * Get settings fields.
	 *
	 * @return array
	 */
	public function get_settings_fields() {
		$fields = array();

		// Get Config ID from Post.
		$config_id = get_the_ID();

		// try to get Config ID from Referer URL if config id not available in Post.
		if ( empty( $config_id ) ) {
			$referer_parameter = array();
			$referer_url       = wp_parse_url( wp_get_referer() );
			parse_str( $referer_url['query'], $referer_parameter );
			$config_id = isset( $referer_parameter['post'] ) ? $referer_parameter['post'] : 0;
		}
		if ( ! empty( $config_id ) ) {
			$this->config = $this->get_config( $config_id );
		}

		$mode = filter_input( INPUT_GET, 'gateway_mode', FILTER_SANITIZE_STRING );

		if ( ! isset( $this->config )
			|| empty( $this->config->access_token )
			|| ( isset( $mode ) && ! strpos( $this->config->key_id, $mode ) ) ) {

			// Intro.
			$fields[] = array(
				'section' => 'general',
				'type'    => 'html',
				'html'    => '<p>' . __( '<h1>Limited Period Offer.</h1>' ) . '</p>' .
				'<p>' . __( '0% Razorpay platform fee, for the first three months. Offer valid on the new account created before 15 May 2021.' ) . '</p>' .
				'<br /> <a class="button button-primary button-large" target="_new" href="' . $this->get_url() . 'special-offer"
    		    role="button"><strong>Sign Up Now</strong></a>',
			);

			// Intro.
			$fields[] = array(
				'section' => 'general',
				'type'    => 'html',
				'html'    => '<p><h1>' . __( 'How it works?' ) . '</h1></p>' .
				'<p>' . __( 'To provide a seamless integration experience, Knit Pay has introduced Razorpay Platform Connect. Now you can integrate Razorpay in Knit Pay with just a few clicks.' ) . '</p>' .
				'<p>' . __( 'Click on "<strong>Connect with Razorpay</strong>" below to initiate the connection.' ) . '</p>',
			);

			// Connect.
			$fields[] = array(
				'section' => 'general',
				'type'    => 'html',
				'html'    => '<a id="razorpay-platform-connect" class="button button-primary button-large"
		                  role="button" style="font-size: 21px;background: #3395ff;">Connect with <strong>Razorpay</strong></a>
                        <script>
                            document.getElementById("razorpay-platform-connect").addEventListener("click", function(event){
                                event.preventDefault();
                                document.getElementById("publish").click();;
                            });
                        </script>',
			);
		} else {

			// Merchant/Company Name.
			$fields[] = array(
				'section'  => 'general',
				'filter'   => FILTER_SANITIZE_STRING,
				'meta_key' => '_pronamic_gateway_razorpay_company_name',
				'title'    => __( 'Merchant/Company Name', 'knit-pay' ),
				'type'     => 'text',
				'classes'  => array( 'regular-text', 'code' ),
				'tooltip'  => __( 'The merchant/company name shown in the Checkout form.', 'knit-pay' ),
			);

			// Checkout Image.
			$fields[] = array(
				'section'  => 'general',
				'filter'   => FILTER_SANITIZE_STRING,
				'meta_key' => '_pronamic_gateway_razorpay_checkout_image',
				'title'    => __( 'Checkout Image', 'knit-pay' ),
				'type'     => 'text',
				'classes'  => array( 'large-text', 'code' ),
				'tooltip'  => __( 'Link to an image (usually your business logo) shown in the Checkout form. Can also be a base64 string, if loading the image from a network is not desirable. Keep it blank to use default image.', 'knit-pay' ),
			);

			// Remove Knit Pay as an Authorized Application.
			$fields[] = array(
				'section'     => 'general',
				'title'       => __( 'Remove Knit Pay as an Connected Application for my Razorpay account.', 'knit-pay' ),
				'type'        => 'description',
				'description' => '<p>Removing Knit Pay as an Connected Application for your Razorpay account will remove the connection between all the sites that you have connected to Knit Pay using the same Razorpay account and connect method. Proceed with caution while disconnecting if you have multiple sites connected.</p>' .
				'<br><a class="button button-primary button-large" target="_new" href="https://dashboard.razorpay.com/app/applications" role="button"><strong>View connected applications in Razorpay</strong></a>',
			);

			// Connected with Razorpay.
			$fields[] = array(
				'section'     => 'general',
				'filter'      => FILTER_VALIDATE_BOOLEAN,
				'meta_key'    => '_pronamic_gateway_razorpay_is_connected',
				'title'       => __( 'Connected with Razorpay', 'knit-pay' ),
				'type'        => 'checkbox',
				'description' => 'This gateway configuration is connected with Razorpay Platform Connect. Uncheck this and save the configuration to disconnect it.',
				'label'       => __( 'Uncheck and save to disconnect the Razorpay Account.', 'knit-pay' ),
			);
		}

		// Webhook URL.
		$fields[] = array(
			'section'  => 'feedback',
			'title'    => \__( 'Webhook URL', 'knit-pay' ),
			'type'     => 'text',
			'classes'  => array( 'large-text', 'code' ),
			'value'    => add_query_arg( 'kp_razorpay_webhook', '', home_url( '/' ) ),
			'readonly' => true,
			'tooltip'  => sprintf(
				/* translators: %s: PayUmoney */
				__(
					'Copy the Webhook URL to the %s dashboard to receive automatic transaction status updates.',
					'knit-pay'
				),
				__( 'Razorpay', 'knit-pay' )
			),
		);

		$fields[] = array(
			'section'  => 'feedback',
			'filter'   => FILTER_SANITIZE_STRING,
			'meta_key' => '_pronamic_gateway_razorpay_webhook_secret',
			'title'    => \__( 'Webhook Secret', 'knit-pay' ),
			'type'     => 'text',
			'classes'  => array( 'regular-text', 'code' ),
			'tooltip'  =>
			__(
				'Create a new webhook secret. This can be a random string, and you don\'t have to remember it. Do not use your password or Key Secret here.',
				'knit-pay'
			),
		);

		$fields[] = array(
			'section' => 'feedback',
			'title'   => \__( 'Active Events', 'knit-pay' ),
			'type'    => 'html',
			'html'    => sprintf(
				/* translators: 1: Razorpay */
				__( 'In Active Events section check payment authorized, failed and captured events.', 'knit-pay' ),
				__( 'Razorpay', 'knit-pay' )
			),
		);

		// Return fields.
		return $fields;
	}

	public function get_config( $post_id ) {
		$config = new Config();

		$config->key_id         = $this->get_meta( $post_id, 'razorpay_key_id' );
		$config->key_secret     = $this->get_meta( $post_id, 'razorpay_key_secret' );
		$config->webhook_secret = $this->get_meta( $post_id, 'razorpay_webhook_secret' );
		$config->is_connected   = $this->get_meta( $post_id, 'razorpay_is_connected' );
		$config->expires_at     = $this->get_meta( $post_id, 'razorpay_expires_at' );
		$config->access_token   = $this->get_meta( $post_id, 'razorpay_access_token' );
		$config->refresh_token  = $this->get_meta( $post_id, 'razorpay_refresh_token' );
		$config->company_name   = $this->get_meta( $post_id, 'razorpay_company_name' );
		$config->checkout_image = $this->get_meta( $post_id, 'razorpay_checkout_image' );
		$config->merchant_id    = $this->get_meta( $post_id, 'razorpay_merchant_id' );
		$config->mode           = $this->get_meta( $post_id, 'mode' );

		// Schedule next refresh token if not done before.
		self::schedule_next_refresh_access_token( $post_id, $config->expires_at );

		return $config;
	}

	/**
	 * Get gateway.
	 *
	 * @param int $post_id Post ID.
	 * @return Gateway
	 */
	public function get_gateway( $post_id ) {
		return new Gateway( $this->get_config( $post_id ) );
	}

	/**
	 * When the post is saved, saves our custom data.
	 *
	 * @param int $config_id The ID of the post being saved.
	 * @return void
	 */
	public function save_post( $config_id ) {
		parent::save_post( $config_id );
		$config = $this->get_config( $config_id );

		if ( empty( $config->access_token ) || ! strpos( $config->key_id, $config->mode ) ) {
			$this->connect( $config, $config_id );
			return;
		}

		// Clear Keys if not connected.
		if ( ! $config->is_connected && ! empty( $config->access_token ) ) {
			self::clear_config( $config_id );
		}
	}

	private function connect( $config, $config_id ) {

		// Clear Old config before creating new connection.
		self::clear_config( $config_id );

		$response = wp_remote_post(
			self::KNIT_PAY_RAZORPAY_PLATFORM_CONNECT_URL,
			array(
				'body'    => array(
					'admin_url'  => rawurlencode( admin_url() ),
					'action'     => 'connect',
					'gateway_id' => $config_id,
					'mode'       => $config->mode,
				),
				'timeout' => 60,
			)
		);
		$result   = wp_remote_retrieve_body( $response );
		$result   = json_decode( $result );
		if ( isset( $result->error ) ) {
			echo $result->error;
			exit;
		}
		if ( isset( $result->return_url ) ) {
			wp_redirect( $result->return_url . '&redirect_uri=' . self::KNIT_PAY_RAZORPAY_PLATFORM_CONNECT_URL, 303 );
			exit;
		}
	}

	private static function clear_config( $config_id ) {
		delete_post_meta( $config_id, '_pronamic_gateway_razorpay_key_id' );
		delete_post_meta( $config_id, '_pronamic_gateway_razorpay_key_secret' );
		delete_post_meta( $config_id, '_pronamic_gateway_razorpay_is_connected' );
		delete_post_meta( $config_id, '_pronamic_gateway_razorpay_expires_at' );
		delete_post_meta( $config_id, '_pronamic_gateway_razorpay_access_token' );
		delete_post_meta( $config_id, '_pronamic_gateway_razorpay_refresh_token' );
		delete_post_meta( $config_id, '_pronamic_gateway_razorpay_merchant_id' );
	}

	public static function update_connection_status() {

		if ( ! ( filter_has_var( INPUT_GET, 'razorpay_connect_status' ) && current_user_can( 'manage_options' ) ) ) {
			return;
		}

		$code                    = filter_input( INPUT_GET, 'code', FILTER_SANITIZE_STRING );
		$state                   = filter_input( INPUT_GET, 'state', FILTER_SANITIZE_STRING );
		$gateway_id              = filter_input( INPUT_GET, 'gateway_id', FILTER_SANITIZE_STRING );
		$razorpay_connect_status = filter_input( INPUT_GET, 'razorpay_connect_status', FILTER_SANITIZE_STRING );

		if ( empty( $code ) || empty( $state ) || 'failed' === $razorpay_connect_status ) {
			self::clear_config( $gateway_id );
			self::redirect_to_config( $gateway_id );
		}

		// GET keys.
		$response = wp_remote_post(
			self::KNIT_PAY_RAZORPAY_PLATFORM_CONNECT_URL,
			array(
				'body'    => array(
					'code'       => $code,
					'state'      => $state,
					'gateway_id' => $gateway_id,
					'action'     => 'get-keys',
				),
				'timeout' => 120,
			)
		);
		$result   = wp_remote_retrieve_body( $response );
		$result   = json_decode( $result );

		self::save_token( $gateway_id, $result );

		self::redirect_to_config( $gateway_id );
	}

	public function refresh_access_token( $config_id ) {
		if ( 'publish' !== get_post_status( $config_id ) ) {
			return;
		}
		$config = $this->get_config( $config_id );

		if ( empty( $config->refresh_token ) ) {
			// Clear All configurations if Refresh Token is missing.
			self::clear_config( $config_id );
			return;
		}

		$time_left_before_expire = $config->expires_at - time();
		if ( $time_left_before_expire > 0 && $time_left_before_expire > self::RENEWAL_TIME_BEFORE_TOKEN_EXPIRE + 432000 ) {
			self::schedule_next_refresh_access_token( $config_id, $config->expires_at );
			return;
		}

		// GET keys.
		$response = wp_remote_post(
			self::KNIT_PAY_RAZORPAY_PLATFORM_CONNECT_URL,
			array(
				'body'    => array(
					'refresh_token' => $config->refresh_token,
					'mode'          => $config->mode,
					'action'        => 'refresh-access-token',
				),
				'timeout' => 120,
			)
		);
		$result   = wp_remote_retrieve_body( $response );
		$result   = json_decode( $result );

		if ( JSON_ERROR_NONE !== json_last_error() ) {
			self::schedule_next_refresh_access_token( $config_id, $config->expires_at );
			return;
		}

		self::save_token( $config_id, $result );
	}

	private static function save_token( $gateway_id, $token_data ) {
		if ( ! ( isset( $token_data->razorpay_connect_status ) && 'connected' === $token_data->razorpay_connect_status ) || empty( $token_data->expires_in ) ) {
			return;
		}

		$expires_at = time() + $token_data->expires_in;

		update_post_meta( $gateway_id, '_pronamic_gateway_razorpay_key_id', $token_data->public_token );
		update_post_meta( $gateway_id, '_pronamic_gateway_razorpay_access_token', $token_data->access_token );
		update_post_meta( $gateway_id, '_pronamic_gateway_razorpay_refresh_token', $token_data->refresh_token );
		update_post_meta( $gateway_id, '_pronamic_gateway_razorpay_expires_at', $expires_at );
		update_post_meta( $gateway_id, '_pronamic_gateway_razorpay_is_connected', true );

		if ( isset( $token_data->merchant_id ) ) {
			update_post_meta( $gateway_id, '_pronamic_gateway_razorpay_merchant_id', $token_data->merchant_id );
		}

		self::schedule_next_refresh_access_token( $gateway_id, $expires_at );
	}

	private static function redirect_to_config( $gateway_id ) {
		wp_safe_redirect( get_edit_post_link( $gateway_id, false ) );
		exit;
	}

	private static function schedule_next_refresh_access_token( $gateway_id, $expires_at ) {
		if ( empty( $expires_at ) ) {
			return;
		}

		if ( wp_next_scheduled( 'knit_pay_razorpay_refresh_access_token', array( 'config_id' => $gateway_id ) ) ) {
			return;
		}

		$next_schedule_time = $expires_at - self::RENEWAL_TIME_BEFORE_TOKEN_EXPIRE + rand( 0, 86400 );
		$current_time       = time();
		if ( $next_schedule_time <= $current_time ) {
			$next_schedule_time = $current_time + rand( 0, 86400 );
		}

		wp_schedule_single_event(
			$next_schedule_time,
			'knit_pay_razorpay_refresh_access_token',
			array( 'config_id' => $gateway_id )
		);
	}
}
